@@ -1,27 +1,62 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * emit.cxx
  *
  * Useful functions for emitting various pieces of code.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_emit_cxx[] = "$Id: emit.cxx 10265 2008-02-25 22:07:28Z wsfulton $";
+char cvsroot_emit_cxx[] = "$Id: emit.cxx 11980 2010-04-08 06:46:18Z wsfulton $";
 
 #include "swigmod.h"
 
 /* -----------------------------------------------------------------------------
- * emit_args()
+ * emit_return_variable()
  *
- * Creates a list of variable declarations for both the return value
- * and function parameters.
- *
- * The return value is always called result and arguments arg0, arg1, arg2, etc...
- * Returns the number of parameters associated with a function.
+ * Emits a variable declaration for a function return value.
+ * The variable name is always called result.
+ * n => Node of the method being wrapped
+ * rt => the return type
+ * f => the wrapper to generate code into
  * ----------------------------------------------------------------------------- */
 
-void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
+void emit_return_variable(Node *n, SwigType *rt, Wrapper *f) {
+
+  if (!GetFlag(n, "tmap:out:optimal")) {
+    if (rt && (SwigType_type(rt) != T_VOID)) {
+      SwigType *vt = cplus_value_type(rt);
+      SwigType *tt = vt ? vt : rt;
+      SwigType *lt = SwigType_ltype(tt);
+      String *lstr = SwigType_str(lt, "result");
+      if (SwigType_ispointer(lt)) {
+        Wrapper_add_localv(f, "result", lstr, "= 0", NULL);
+      } else {
+        Wrapper_add_local(f, "result", lstr);
+      }
+      if (vt) {
+        Delete(vt);
+      }
+      Delete(lt);
+      Delete(lstr);
+    }
+  }
+}
+
+/* -----------------------------------------------------------------------------
+ * emit_parameter_variables()
+ *
+ * Emits a list of variable declarations for function parameters.
+ * The variable names are always called arg1, arg2, etc...
+ * l => the parameter list
+ * f => the wrapper to generate code into
+ * ----------------------------------------------------------------------------- */
+
+void emit_parameter_variables(ParmList *l, Wrapper *f) {
 
   Parm *p;
   String *tm;
@@ -29,24 +64,6 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
   /* Emit function arguments */
   Swig_cargs(f, l);
 
-  /* Handle return type */
-  if (rt && (SwigType_type(rt) != T_VOID)) {
-    SwigType *vt = cplus_value_type(rt);
-    SwigType *tt = vt ? vt : rt;
-    SwigType *lt = SwigType_ltype(tt);
-    String *lstr = SwigType_str(lt, "result");
-    if (SwigType_ispointer(lt)) {
-      Wrapper_add_localv(f, "result", lstr, "= 0", NULL);
-    } else {
-      Wrapper_add_local(f, "result", lstr);
-    }
-    if (vt) {
-      Delete(vt);
-    }
-    Delete(lt);
-    Delete(lstr);
-  }
-
   /* Attach typemaps to parameters */
   /*  Swig_typemap_attach_parms("ignore",l,f); */
 
@@ -78,7 +95,6 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
       p = nextSibling(p);
     }
   }
-  return;
 }
 
 /* -----------------------------------------------------------------------------
@@ -188,7 +204,7 @@ void emit_attach_parmmaps(ParmList *l, Wrapper *f) {
  * emit_num_arguments()
  *
  * Calculate the total number of arguments.   This function is safe for use
- * with multi-valued typemaps which may change the number of arguments in
+ * with multi-argument typemaps which may change the number of arguments in
  * strange ways.
  * ----------------------------------------------------------------------------- */
 
@@ -220,7 +236,7 @@ int emit_num_arguments(ParmList *parms) {
  * emit_num_required()
  *
  * Computes the number of required arguments.  This function is safe for
- * use with multi-valued typemaps and knows how to skip over everything
+ * use with multi-argument typemaps and knows how to skip over everything
  * properly. Note that parameters with default values are counted unless
  * the compact default args option is on.
  * ----------------------------------------------------------------------------- */
@@ -332,14 +348,16 @@ static void replace_contract_args(Parm *cp, Parm *rp, String *s) {
 #endif
 
 /* -----------------------------------------------------------------------------
- * int emit_action()
+ * int emit_action_code()
  *
- * Emits action code for a wrapper and checks for exception handling
+ * Emits action code for a wrapper. Adds in exception handling code (%exception).
+ * eaction -> the action code to emit
+ * wrappercode -> the emitted code (output)
  * ----------------------------------------------------------------------------- */
-int emit_action_code(Node *n, Wrapper *f, String *eaction) {
+int emit_action_code(Node *n, String *wrappercode, String *eaction) {
   assert(Getattr(n, "wrap:name"));
 
-  /* Look for except feature */
+  /* Look for except feature (%exception) */
   String *tm = GetFlagAttr(n, "feature:except");
   if (tm)
     tm = Copy(tm);
@@ -364,16 +382,23 @@ int emit_action_code(Node *n, Wrapper *f, String *eaction) {
         Delete(fulldecl);
       }
     }
-    Printv(f->code, tm, "\n", NIL);
+    Printv(wrappercode, tm, "\n", NIL);
     Delete(tm);
     return 1;
   } else {
-    Printv(f->code, eaction, "\n", NIL);
+    Printv(wrappercode, eaction, "\n", NIL);
     return 0;
   }
 }
 
-void emit_action(Node *n, Wrapper *f) {
+/* -----------------------------------------------------------------------------
+ * int emit_action()
+ *
+ * Emits the call to the wrapped function. 
+ * Adds in exception specification exception handling and %exception code.
+ * ----------------------------------------------------------------------------- */
+String *emit_action(Node *n) {
+  String *actioncode = NewStringEmpty();
   String *tm;
   String *action;
   String *wrap;
@@ -382,11 +407,10 @@ void emit_action(Node *n, Wrapper *f) {
 
   /* Look for fragments */
   {
-    String *f;
-    f = Getattr(n, "feature:fragment");
-    if (f) {
+    String *fragment = Getattr(n, "feature:fragment");
+    if (fragment) {
       char *c, *tok;
-      String *t = Copy(f);
+      String *t = Copy(fragment);
       c = Char(t);
       tok = strtok(c, ",");
       while (tok) {
@@ -410,26 +434,13 @@ void emit_action(Node *n, Wrapper *f) {
     }
     Setattr(n, "wrap:code:done", f_code);
   }
+
   action = Getattr(n, "feature:action");
   if (!action)
     action = Getattr(n, "wrap:action");
   assert(action != 0);
 
-  if (!is_public(n) && (is_member_director(n) || GetFlag(n, "explicitcall"))) {
-    /* In order to call protected virtual director methods from the target language, we need
-     * to add an extra dynamic_cast to call the public C++ wrapper in the director class. */
-    Node *parent = Getattr(n, "parentNode");
-    String *symname = Getattr(parent, "sym:name");
-    String *dirname = NewStringf("SwigDirector_%s", symname);
-    String *dirdecl = NewStringf("%s *darg = 0", dirname);
-    Wrapper_add_local(f, "darg", dirdecl);
-    Printf(f->code, "darg = dynamic_cast<%s *>(arg1);\n", dirname);
-    Delete(dirname);
-    Delete(dirdecl);
-  }
-
   /* Get the return type */
-
   rt = Getattr(n, "type");
 
   /* Emit contract code (if any) */
@@ -437,7 +448,7 @@ void emit_action(Node *n, Wrapper *f) {
     /* Preassertion */
     tm = Getattr(n, "contract:preassert");
     if (Len(tm)) {
-      Printv(f->code, tm, "\n", NIL);
+      Printv(actioncode, tm, "\n", NIL);
     }
   }
   /* Exception handling code */
@@ -451,44 +462,52 @@ void emit_action(Node *n, Wrapper *f) {
     Printf(eaction, "try {\n");
   }
 
+  String *preaction = Getattr(n, "wrap:preaction");
+  if (preaction)
+    Printv(eaction, preaction, NIL);
+
   Printv(eaction, action, NIL);
 
+  String *postaction = Getattr(n, "wrap:postaction");
+  if (postaction)
+    Printv(eaction, postaction, NIL);
+
   if (catchlist) {
     int unknown_catch = 0;
     Printf(eaction, "}\n");
     for (Parm *ep = catchlist; ep; ep = nextSibling(ep)) {
-      String *em = Swig_typemap_lookup_new("throws", ep, "_e", 0);
+      String *em = Swig_typemap_lookup("throws", ep, "_e", 0);
       if (em) {
-	SwigType *et = Getattr(ep, "type");
-	SwigType *etr = SwigType_typedef_resolve_all(et);
-	if (SwigType_isreference(etr) || SwigType_ispointer(etr) || SwigType_isarray(etr)) {
-	  Printf(eaction, "catch(%s) {", SwigType_str(et, "_e"));
-	} else if (SwigType_isvarargs(etr)) {
-	  Printf(eaction, "catch(...) {");
-	} else {
-	  Printf(eaction, "catch(%s) {", SwigType_str(et, "&_e"));
-	}
-	Printv(eaction, em, "\n", NIL);
-	Printf(eaction, "}\n");
+        SwigType *et = Getattr(ep, "type");
+        SwigType *etr = SwigType_typedef_resolve_all(et);
+        if (SwigType_isreference(etr) || SwigType_ispointer(etr) || SwigType_isarray(etr)) {
+          Printf(eaction, "catch(%s) {", SwigType_str(et, "_e"));
+        } else if (SwigType_isvarargs(etr)) {
+          Printf(eaction, "catch(...) {");
+        } else {
+          Printf(eaction, "catch(%s) {", SwigType_str(et, "&_e"));
+        }
+        Printv(eaction, em, "\n", NIL);
+        Printf(eaction, "}\n");
       } else {
 	Swig_warning(WARN_TYPEMAP_THROW, Getfile(n), Getline(n), "No 'throws' typemap defined for exception type '%s'\n", SwigType_str(Getattr(ep, "type"), 0));
-	unknown_catch = 1;
+        unknown_catch = 1;
       }
     }
     if (unknown_catch) {
-      Printf(eaction, "catch(...) { throw; }\n");
+    Printf(eaction, "catch(...) { throw; }\n");
     }
   }
 
   /* Look for except typemap (Deprecated) */
-  tm = Swig_typemap_lookup_new("except", n, "result", 0);
+  tm = Swig_typemap_lookup("except", n, "result", 0);
   if (tm) {
     Setattr(n, "feature:except", tm);
     tm = 0;
   }
 
   /* emit the except feature code */
-  emit_action_code(n, f, eaction);
+  emit_action_code(n, actioncode, eaction);
 
   Delete(eaction);
 
@@ -497,8 +516,9 @@ void emit_action(Node *n, Wrapper *f) {
     /* Postassertion */
     tm = Getattr(n, "contract:postassert");
     if (Len(tm)) {
-      Printv(f->code, tm, "\n", NIL);
+      Printv(actioncode, tm, "\n", NIL);
     }
   }
 
+  return actioncode;
 }
