@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * lua.cxx
  *
@@ -40,7 +44,7 @@
  
 */
 
-char cvsroot_lua_cxx[] = "$Id: lua.cxx 10326 2008-03-27 01:58:30Z mgossage $";
+char cvsroot_lua_cxx[] = "$Id: lua.cxx 12320 2010-11-29 07:49:42Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -51,6 +55,7 @@ char cvsroot_lua_cxx[] = "$Id: lua.cxx 10326 2008-03-27 01:58:30Z mgossage $";
 */
 #define REPORT(T,D)		// no info:
 //#define REPORT(T,D)   {Printf(stdout,T"\n");} // only title
+//#define REPORT(T,D)		{Printf(stdout,T" %p\n",n);} // title & pointer
 //#define REPORT(T,D)   {Printf(stdout,T"\n");display_mapping(D);}      // the works
 //#define REPORT(T,D)   {Printf(stdout,T"\n");if(D)Swig_print_node(D);}      // the works
 
@@ -91,6 +96,7 @@ NEW LANGUAGE NOTE:END ************************************************/
 class LUA:public Language {
 private:
 
+  File *f_begin;
   File *f_runtime;
   File *f_header;
   File *f_wrappers;
@@ -131,6 +137,7 @@ public:
    * --------------------------------------------------------------------- */
 
   LUA() {
+    f_begin = 0;
     f_runtime = 0;
     f_header = 0;
     f_wrappers = 0;
@@ -212,11 +219,12 @@ public:
     String *outfile = Getattr(n, "outfile");
 
     /* Open the output file */
-    f_runtime = NewFile(outfile, "w");
-    if (!f_runtime) {
+    f_begin = NewFile(outfile, "w", SWIG_output_files());
+    if (!f_begin) {
       FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
+    f_runtime = NewString("");
     f_init = NewString("");
     f_header = NewString("");
     f_wrappers = NewString("");
@@ -225,6 +233,7 @@ public:
     /* Register file targets with the SWIG file handler */
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("wrapper", f_wrappers);
+    Swig_register_filebyname("begin", f_begin);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("init", f_init);
     Swig_register_filebyname("initbeforefunc", f_initbeforefunc);
@@ -248,11 +257,17 @@ public:
     current=NO_CPP;
 
     /* Standard stuff for the SWIG runtime section */
-    Swig_banner(f_runtime);
+    Swig_banner(f_begin);
+
+    Printf(f_runtime, "\n");
+    Printf(f_runtime, "#define SWIGLUA\n");
+
     //    if (NoInclude) {
     //      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
     //    }
 
+    Printf(f_runtime, "\n");
+
     //String *init_name = NewStringf("%(title)s_Init", module);
     //Printf(f_header, "#define SWIG_init    %s\n", init_name);
     //Printf(f_header, "#define SWIG_name    \"%s\"\n", module);
@@ -287,13 +302,14 @@ public:
      this basically combines several of the strings together
      and then writes it all to a file
     NEW LANGUAGE NOTE:END ************************************************/
-    Dump(f_header, f_runtime);
-    Dump(f_wrappers, f_runtime);
-    Dump(f_initbeforefunc, f_runtime);
+    Dump(f_runtime, f_begin);
+    Dump(f_header, f_begin);
+    Dump(f_wrappers, f_begin);
+    Dump(f_initbeforefunc, f_begin);
     /* for the Lua code it needs to be properly excaped to be added into the C/C++ code */
     EscapeCode(s_luacode);
-    Printf(f_runtime, "const char* SWIG_LUACODE=\n  \"%s\";\n\n",s_luacode);
-    Wrapper_pretty_print(f_init, f_runtime);
+    Printf(f_begin, "const char* SWIG_LUACODE=\n  \"%s\";\n\n",s_luacode);
+    Wrapper_pretty_print(f_init, f_begin);
     /* Close all of the files */
     Delete(s_luacode);
     Delete(s_cmd_tab);
@@ -303,8 +319,9 @@ public:
     Delete(f_wrappers);
     Delete(f_init);
     Delete(f_initbeforefunc);
-    Close(f_runtime);
+    Close(f_begin);
     Delete(f_runtime);
+    Delete(f_begin);
 
     /* Done */
     return SWIG_OK;
@@ -333,7 +350,7 @@ public:
    * --------------------------------------------------------------------- */
 
   virtual int functionWrapper(Node *n) {
-    //    REPORT("functionWrapper",n);
+    REPORT("functionWrapper",n);
 
     String *name = Getattr(n, "name");
     String *iname = Getattr(n, "sym:name");
@@ -361,7 +378,7 @@ public:
         we need to add a couple of local variables
     NEW LANGUAGE NOTE:END ************************************************/
     Wrapper *f = NewWrapper();
-    Wrapper_add_local(f, "SWIG_arg", "int SWIG_arg = -1");
+    Wrapper_add_local(f, "SWIG_arg", "int SWIG_arg = 0");
 
 
     String *wname = Swig_name_wrapper(iname);
@@ -382,10 +399,9 @@ public:
        it will print
          int arg1;
          int arg2;
-         int result;
     NEW LANGUAGE NOTE:END ************************************************/
     /* Write code to extract function parameters. */
-    emit_args(d, l, f);
+    emit_parameter_variables(l, f);
 
     /* Attach the standard typemaps */
     emit_attach_parmmaps(l, f);
@@ -490,8 +506,6 @@ public:
         Swig_warning(WARN_TYPEMAP_IN_UNDEF, input_file, line_number, "Unable to use type %s as a function argument.\n", SwigType_str(pt, 0));
         break;
       }
-
-      p = nextSibling(p);
     }
 
     // add all argcheck code
@@ -553,16 +567,15 @@ public:
     Setattr(n, "wrap:name", wname);
 
     /* Emit the function call */
-    emit_action(n, f);
+    String *actioncode = emit_action(n);
 
     /* NEW LANGUAGE NOTE:***********************************************
     FIXME:
     returns 1 if there is a void return type
     this is because there is a typemap for void
     NEW LANGUAGE NOTE:END ************************************************/
-    Printv(f->code, "SWIG_arg=0;\n", NIL);
     // Return value if necessary
-    if ((tm = Swig_typemap_lookup_new("out", n, "result", 0))) {
+    if ((tm = Swig_typemap_lookup_out("out", n, "result", f, actioncode))) {
       // managing the number of returning variables
       //      if (numoutputs=Getattr(tm,"numoutputs")){
       //              int i=GetInt(tm,"numoutputs");
@@ -581,6 +594,7 @@ public:
     } else {
       Swig_warning(WARN_TYPEMAP_OUT_UNDEF, input_file, line_number, "Unable to use return type %s in function %s.\n", SwigType_str(d, 0), name);
     }
+    emit_return_variable(n, d, f);
 
     /* Output argument output code */
     Printv(f->code, outarg, NIL);
@@ -590,14 +604,14 @@ public:
 
     /* Look to see if there is any newfree cleanup code */
     if (GetFlag(n, "feature:new")) {
-      if ((tm = Swig_typemap_lookup_new("newfree", n, "result", 0))) {
+      if ((tm = Swig_typemap_lookup("newfree", n, "result", 0))) {
         Replaceall(tm, "$source", "result");
         Printf(f->code, "%s\n", tm);
       }
     }
 
     /* See if there is any return cleanup code */
-    if ((tm = Swig_typemap_lookup_new("ret", n, "result", 0))) {
+    if ((tm = Swig_typemap_lookup("ret", n, "result", 0))) {
       Replaceall(tm, "$source", "result");
       Printf(f->code, "%s\n", tm);
     }
@@ -620,8 +634,17 @@ public:
     Replaceall(f->code, "$result", "result");
 
     /* Dump the function out */
-    Wrapper_print(f, f_wrappers);
-
+    /* in Lua we will not emit the destructor as a wrappered function,
+    Lua will automatically call the destructor when the object is free'd
+    However: you cannot just skip this function as it will not emit
+    any custom destructor (using %extend), as you need to call emit_action()
+    Therefore we go though the whole function, 
+    but do not write the code into the wrapper
+    */
+    if(current!=DESTRUCTOR) {
+       Wrapper_print(f, f_wrappers);
+    }
+    
     /* NEW LANGUAGE NOTE:***********************************************
     register the function in SWIG
     different language mappings seem to use different ideas
@@ -689,7 +712,18 @@ public:
 
     Replaceall(dispatch, "$args", "self,args");
     Printv(f->code, dispatch, "\n", NIL);
-    Printf(f->code, "lua_pushstring(L,\"No matching function for overloaded '%s'\");\n", symname);
+    
+    Node *sibl = n;
+    while (Getattr(sibl, "sym:previousSibling"))
+      sibl = Getattr(sibl, "sym:previousSibling");	// go all the way up
+    String *protoTypes = NewString("");
+    do {
+      Printf(protoTypes, "\n\"    %s(%s)\\n\"", SwigType_str(Getattr(sibl, "name"), 0), ParmList_protostr(Getattr(sibl, "wrap:parms")));
+    } while ((sibl = Getattr(sibl, "sym:nextSibling")));
+    Printf(f->code, "lua_pushstring(L,\"Wrong arguments for overloaded function '%s'\\n\"\n"
+        "\"  Possible C/C++ prototypes are:\\n\"%s);\n",symname,protoTypes);
+    Delete(protoTypes);
+
     Printf(f->code, "lua_error(L);return 0;\n");
     Printv(f->code, "}\n", NIL);
     Wrapper_print(f, f_wrappers);
@@ -724,19 +758,19 @@ public:
     current=NO_CPP;
     // normally SWIG will generate 2 wrappers, a get and a set
     // but in certain scenarios (immutable, or if its arrays), it will not
-    String *getName = Swig_name_wrapper(Swig_name_get(iname));
+    String *getName = Swig_name_wrapper(Swig_name_get(NSPACE_TODO, iname));
     String *setName = 0;
     // checking whether it can be set to or not appears to be a very error prone issue
     // I refered to the Language::variableWrapper() to find this out
     bool assignable=is_assignable(n) ? true : false;
     SwigType *type = Getattr(n, "type");
-    String *tm = Swig_typemap_lookup_new("globalin", n, iname, 0);
+    String *tm = Swig_typemap_lookup("globalin", n, iname, 0);
     if (!tm && SwigType_isarray(type))
       assignable=false;
     Delete(tm);
 
     if (assignable) {
-      setName = Swig_name_wrapper(Swig_name_set(iname));
+      setName = Swig_name_wrapper(Swig_name_set(NSPACE_TODO, iname));
     } else {
       // how about calling a 'this is not settable' error message?
       setName = NewString("SWIG_Lua_set_immutable"); // error message
@@ -756,7 +790,6 @@ public:
     //    REPORT("constantWrapper", n);
     String *name = Getattr(n, "name");
     String *iname = Getattr(n, "sym:name");
-    //String *nsname    = !nspace ? Copy(iname) : NewStringf("%s::%s",ns_name,iname);
     String *nsname = Copy(iname);
     SwigType *type = Getattr(n, "type");
     String *rawval = Getattr(n, "rawval");
@@ -765,7 +798,6 @@ public:
 
     if (!addSymbol(iname, n))
       return SWIG_ERROR;
-    //if (nspace) Setattr(n,"sym:name",nsname);
 
     /* Special hook for member pointer */
     if (SwigType_type(type) == T_MPOINTER) {
@@ -774,13 +806,13 @@ public:
       value = Char(wname);
     }
 
-    if ((tm = Swig_typemap_lookup_new("consttab", n, name, 0))) {
+    if ((tm = Swig_typemap_lookup("consttab", n, name, 0))) {
       Replaceall(tm, "$source", value);
       Replaceall(tm, "$target", name);
       Replaceall(tm, "$value", value);
       Replaceall(tm, "$nsname", nsname);
       Printf(s_const_tab, "%s,\n", tm);
-    } else if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
+    } else if ((tm = Swig_typemap_lookup("constcode", n, name, 0))) {
       Replaceall(tm, "$source", value);
       Replaceall(tm, "$target", name);
       Replaceall(tm, "$value", value);
@@ -840,7 +872,7 @@ public:
    * ------------------------------------------------------------ */
 
   virtual int classHandler(Node *n) {
-    REPORT("classHandler", n);
+    //REPORT("classHandler", n);
 
     String *mangled_classname = 0;
     String *real_classname = 0;
@@ -887,16 +919,18 @@ public:
     String *wrap_class = NewStringf("&_wrap_class_%s", mangled_classname);
     SwigType_remember_clientdata(t, wrap_class);
 
-    String *rt = Copy(Getattr(n, "classtype"));
+    String *rt = Copy(getClassType());
     SwigType_add_pointer(rt);
 
     // Register the class structure with the type checker
     //    Printf(f_init,"SWIG_TypeClientData(SWIGTYPE%s, (void *) &_wrap_class_%s);\n", SwigType_manglestr(t), mangled_classname);
+ 
+    // emit a function to be called to delete the object 
     if (have_destructor) {
       Printv(f_wrappers, "static void swig_delete_", class_name, "(void *obj) {\n", NIL);
       if (destructor_action) {
         Printv(f_wrappers, SwigType_str(rt, "arg1"), " = (", SwigType_str(rt, 0), ") obj;\n", NIL);
-        Printv(f_wrappers, destructor_action, NIL);
+        Printv(f_wrappers, destructor_action, "\n", NIL);
       } else {
         if (CPlusPlus) {
           Printv(f_wrappers, "    delete (", SwigType_str(rt, 0), ") obj;\n", NIL);
@@ -961,7 +995,7 @@ public:
     Printv(f_wrappers, "static swig_lua_class _wrap_class_", mangled_classname, " = { \"", class_name, "\", &SWIGTYPE", SwigType_manglestr(t), ",", NIL);
 
     if (have_constructor) {
-      Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(constructor_name)));
+      Printf(f_wrappers, "%s", Swig_name_wrapper(Swig_name_construct(NSPACE_TODO, constructor_name)));
       Delete(constructor_name);
       constructor_name = 0;
     } else {
@@ -1007,7 +1041,7 @@ public:
     current = NO_CPP;
 
     realname = iname ? iname : name;
-    rname = Swig_name_wrapper(Swig_name_member(class_name, realname));
+    rname = Swig_name_wrapper(Swig_name_member(NSPACE_TODO, class_name, realname));
     if (!Getattr(n, "sym:nextSibling")) {
       Printv(s_methods_tab, tab4, "{\"", realname, "\", ", rname, "}, \n", NIL);
     }
@@ -1027,9 +1061,9 @@ public:
     current = MEMBER_VAR;
     Language::membervariableHandler(n);
     current = NO_CPP;
-    gname = Swig_name_wrapper(Swig_name_get(Swig_name_member(class_name, symname)));
+    gname = Swig_name_wrapper(Swig_name_get(NSPACE_TODO, Swig_name_member(NSPACE_TODO, class_name, symname)));
     if (!GetFlag(n, "feature:immutable")) {
-      sname = Swig_name_wrapper(Swig_name_set(Swig_name_member(class_name, symname)));
+      sname = Swig_name_wrapper(Swig_name_set(NSPACE_TODO, Swig_name_member(NSPACE_TODO, class_name, symname)));
     } else {
       //sname = NewString("0");
       sname = NewString("SWIG_Lua_set_immutable"); // error message
@@ -1061,7 +1095,7 @@ public:
    * ------------------------------------------------------------ */
 
   virtual int destructorHandler(Node *n) {
-    //REPORT("destructorHandler", n);
+    REPORT("destructorHandler", n);
     current = DESTRUCTOR;
     Language::destructorHandler(n);
     current = NO_CPP;
@@ -1079,7 +1113,6 @@ public:
   virtual int staticmemberfunctionHandler(Node *n) {
     current = STATIC_FUNC;
     return Language::staticmemberfunctionHandler(n);
-    current = NO_CPP;
   }
 
   /* ------------------------------------------------------------
@@ -1101,7 +1134,6 @@ public:
     //    REPORT("staticmembervariableHandler",n);
     current = STATIC_VAR;
     return Language::staticmembervariableHandler(n);
-    current = NO_CPP;
   }
 
   /* ---------------------------------------------------------------------
