@@ -13,7 +13,7 @@
  * building a dispatch function.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_overload_cxx[] = "$Id: overload.cxx 11876 2010-02-27 23:53:33Z wsfulton $";
+char cvsroot_overload_cxx[] = "$Id: overload.cxx 12474 2011-02-18 21:25:42Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -59,7 +59,7 @@ void Wrapper_cast_dispatch_mode_set(int flag) {
  * languages ignore the first method parsed.
  * ----------------------------------------------------------------------------- */
 
-static List *Swig_overload_rank(Node *n, bool script_lang_wrapping) {
+List *Swig_overload_rank(Node *n, bool script_lang_wrapping) {
   Overloaded nodes[MAX_OVERLOAD];
   int nnodes = 0;
   Node *o = Getattr(n, "sym:overloaded");
@@ -386,16 +386,11 @@ String *Swig_overload_dispatch_cast(Node *n, const_String_or_char_ptr fmt, int *
     int num_arguments = emit_num_arguments(pi);
     if (num_arguments > *maxargs)
       *maxargs = num_arguments;
-    int varargs = emit_isvarargs(pi);
 
-    if (!varargs) {
-      if (num_required == num_arguments) {
-	Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
-      } else {
-	Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
-      }
+    if (num_required == num_arguments) {
+      Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
     } else {
-      Printf(f, "if (%s >= %d) {\n", argc_template_string, num_required);
+      Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
     }
     Printf(f, "SWIG_TypeRank _ranki = 0;\n");
     Printf(f, "SWIG_TypeRank _rankm = 0;\n");
@@ -565,16 +560,11 @@ String *Swig_overload_dispatch_fast(Node *n, const_String_or_char_ptr fmt, int *
     int num_arguments = emit_num_arguments(pi);
     if (num_arguments > *maxargs)
       *maxargs = num_arguments;
-    int varargs = emit_isvarargs(pi);
 
-    if (!varargs) {
-      if (num_required == num_arguments) {
-	Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
-      } else {
-	Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
-      }
+    if (num_required == num_arguments) {
+      Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
     } else {
-      Printf(f, "if (%s >= %d) {\n", argc_template_string, num_required);
+      Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
     }
 
     /* create a list with the wrappers that collide with the
@@ -732,16 +722,11 @@ String *Swig_overload_dispatch(Node *n, const_String_or_char_ptr fmt, int *maxar
     }
     if (num_arguments > *maxargs)
       *maxargs = num_arguments;
-    int varargs = emit_isvarargs(pi);
 
-    if (!varargs) {
-      if (num_required == num_arguments) {
-	Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
-      } else {
-	Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
-      }
+    if (num_required == num_arguments) {
+      Printf(f, "if (%s == %d) {\n", argc_template_string, num_required);
     } else {
-      Printf(f, "if (%s >= %d) {\n", argc_template_string, num_required);
+      Printf(f, "if ((%s >= %d) && (%s <= %d)) {\n", argc_template_string, num_required, argc_template_string, num_arguments);
     }
 
     if (num_arguments) {
