@@ -11,7 +11,7 @@
  * R language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_r_cxx[] = "$Id: r.cxx 12023 2010-05-12 04:12:42Z drjoe $";
+char cvsroot_r_cxx[] = "$Id: r.cxx 12432 2011-02-04 23:42:06Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -300,7 +300,7 @@ public:
       Printf(stderr, "<memberfunctionHandler> %s %s\n", 
 	     Getattr(n, "name"),
 	     Getattr(n, "type"));
-    member_name = Getattr(n, "name");
+    member_name = Getattr(n, "sym:name");
     processing_class_member_function = 1;
     int status = Language::memberfunctionHandler(n);    
     processing_class_member_function = 0;
@@ -1299,6 +1299,8 @@ void R::addAccessor(String *memberName, Wrapper *wrapper, String *name,
     Printf(stderr, "Adding accessor: %s (%s) => %s\n", memberName, name, tmp);
 }
 
+#define Swig_overload_rank R_swig_overload_rank
+
 #define MAX_OVERLOAD 256
 
 struct Overloaded {
@@ -1601,6 +1603,16 @@ void R::dispatchFunction(Node *n) {
 	       j == 0 ? "" : " && ",
 	       j+1);
 	}
+	else if (DohStrcmp(tm,"integer")==0) {
+	Printf(f->code, "%s(is.integer(argv[[%d]]) || is.numeric(argv[[%d]]))",
+	       j == 0 ? "" : " && ",
+	       j+1, j+1);
+	}
+	else if (DohStrcmp(tm,"character")==0) {
+	Printf(f->code, "%sis.character(argv[[%d]])",
+	       j == 0 ? "" : " && ",
+	       j+1);
+	}
 	else {
 	Printf(f->code, "%sextends(argtypes[%d], '%s')",
 	       j == 0 ? "" : " && ",
@@ -1615,7 +1627,10 @@ void R::dispatchFunction(Node *n) {
     }
   }
   if (cur_args != -1) {
-    Printv(f->code, "}", NIL);
+    Printf(f->code, "} else {\n"
+	   "stop(\"cannot find overloaded function for %s with argtypes (\","
+	   "toString(argtypes),\")\");\n"
+	   "}", sfname);
   }
   Printv(f->code, ";\nf(...)", NIL);
   Printv(f->code, ";\n}", NIL);
@@ -2512,7 +2527,7 @@ int R::membervariableHandler(Node *n) {
 
   int status(Language::membervariableHandler(n));
 
-  if(opaqueClassDeclaration == NULL && debugMode)
+  if(!opaqueClassDeclaration && debugMode)
     Printf(stderr, "<membervariableHandler> %s %s\n", Getattr(n, "name"), Getattr(n, "type"));
 
   processing_member_access_function = 0;
